import os

from kafka import KafkaConsumer
import pkg_resources
import logging

ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)



def get_consumer(topic) -> KafkaConsumer:
    consumer = KafkaConsumer(topic,
                             bootstrap_servers=os.getenv(
                                 'KAFKA_SUBSCRIBER', 'localhost:9093'),
                             group_id=os.getenv(
                                 "OTEL_SERVICE_NAME")
                             )

    return consumer


import base64
import json
import logging
import os

import requests
import pkg_resources
import logging

ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)



def get_future_assets():
    asset_list_raw = requests.get(
        'https://fapi.binance.com/fapi/v1/ticker/price').json()

    asset_list_full = []
    for item in asset_list_raw:
        if (item['symbol'][-4:] == 'USDT' or item['symbol'][-4:] == 'BUSD'):
            # print(item)
            asset_list_full.append(item["symbol"])

    return asset_list_raw

    
def build_request(ticker, type, price, stop_loss, take_profit, valid_until):
    token = f"{os.environ.get('BINANCE_API_KEY')};{os.environ.get('BINANCE_API_SECRET')}"

    price = float(price)
    stop_loss = float(stop_loss)
    take_profit = float(take_profit)

    data = {
        "token": token,
        "ticker": ticker,
        "type": type,
        "stop_loss": stop_loss,
        "take_profit": take_profit,
        "strategy": "simple_gap_finder",
        "valid_until": valid_until
    }
    return data


def request_it(request) -> str:
    
    request = base64.b64encode(bytes(json.dumps(request), "utf-8"))
    request = request.decode()
    send_data ={"message": {"data": request}}
    send_data = json.dumps(send_data)
    
    
    logging.info(send_data)
    
    resp = requests.post(
                            os.environ.get('BINANCE_ORDERS_ENDPOINT', ""), 
                            data=send_data
                            )
    
    logging.info(resp.text)
    
    return resp.text
    
    
def send_order(ticker, type, price, stop_loss, take_profit, valid_until) -> str:
    req = build_request(ticker, type, price, stop_loss, take_profit, valid_until)
    ans = request_it(req)
    
    return ans


import os

from kafka import KafkaProducer
import pkg_resources
import logging

ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)



def get_producer():
    producer = KafkaProducer(bootstrap_servers=os.getenv(
        'KAFKA_ADDRESS', 'localhost:9092'),
                             client_id=os.getenv(
                                 "OTEL_SERVICE_NAME")
                             )

    return producer


import logging
import os
import pkg_resources

import mysql.connector


ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)

def connect_mysql():
    cnx = mysql.connector.connect(user=os.getenv("MYSQL_CRYPTO_USER"),
                                  password=os.getenv(
        "MYSQL_CRYPTO_PASSWORD"),
        host=os.getenv(
        "MYSQL_CRYPTO_SERVER"),
        database=os.getenv(
        "MYSQL_CRYPTO_DB"),
        connection_timeout=30
    )

    cursor = cnx.cursor(buffered=True)

    return cnx, cursor


def build_sql(record_list, table, mode="REPLACE") -> str:

    sql = f"{mode} INTO `{table}` ("

    keys = record_list[0].keys()

    for key in keys:
        sql += "`" + key + "`, \n"
    sql = sql[:-3]

    sql += ") VALUES "

    for record in record_list:
        sql += "("
        for key in keys:
            if (str(record[key]).lower() in ["nan", "inf"]):
                sql += "NULL"
            else:
                sql += '"' + str(record[key]) + '"'

            sql += ", "

        sql = sql[:-2]
        sql += "), "

    sql = sql[:-2]

    return sql


def update_sql(record_list: list[dict], table: str, mode="REPLACE"):

    logging.info(f"Inserting {len(record_list)} records on {table}")
    cnx, cursor = connect_mysql()
    sql = build_sql(record_list, table, mode)

    cursor.execute(sql)

    cnx.commit()
    cursor.close()
    cnx.close()



import datetime
import pkg_resources
import logging

ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)



def strategy_output(
    ticker,
    timeframe,
    pet_datetime,
    entry_value,
    disruption_value,
    stop_loss,
    take_profit,
    direction
) -> dict:

    if (timeframe == 'm15'):
        minutes = 15
    elif (timeframe == 'm30'):
        minutes = 30
    elif (timeframe == 'h1'):
        minutes = 60
    elif (timeframe == 'd1'):
        minutes = 1440
    else:
        raise

    valid_until = pet_datetime + datetime.timedelta(minutes=minutes)

    ret = {}
    ret['ticker'] = ticker
    ret['datetime'] = pet_datetime
    ret['entry_value'] = entry_value
    ret['disruption_value'] = disruption_value
    ret['stop_loss'] = stop_loss
    ret['take_profit'] = take_profit
    ret['direction'] = direction
    ret['timeframe'] = timeframe
    ret['valid_until'] = valid_until

    return ret



import datetime
import pandas as pd
import pkg_resources
import logging

ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)


def generate_benchmark_list(start: datetime.datetime, 
                            end: datetime.datetime, 
                            period: str):
    
    if period == '5m' or period == 'm5':
        minutes = 5
    elif period == '15m' or period == 'm15':
        minutes = 15
    elif period == '30m' or period == 'm30':
        minutes = 30
    elif period == '1h' or period == 'h1':
        minutes = 60
    else:
        minutes = 0

    datepointer = start        
    division_list = []
    
    while datepointer <= end:
        if(datepointer.minute % minutes == 0):
            division_list.append(datepointer.replace(second=0, microsecond=0))
        
        datepointer = datepointer + datetime.timedelta(minutes=1)
    
    division_list.pop(-1)
    
    return division_list


def check_consistency(start, 
                      end, 
                      actual , 
                      period: str) -> list:
    benchmark = generate_benchmark_list(start, end, period)
    
    lacking = []
    if isinstance(actual, pd.Series):
        actual = actual.to_list()
    
    for item in benchmark:
        if isinstance(item, pd.Timestamp):
            item = item.to_pydatetime()
        if item not in actual:
            lacking.append(item)
            
    return lacking



import logging
import os
from datetime import datetime

import pandas as pd
import pymongo
import pkg_resources

ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)


def get_client() -> pymongo.MongoClient:
    client = pymongo.MongoClient(
        os.getenv(
            'MONGO_URI', 'mongodb://root:9M1RSaFuut@localhost:27017'),
        readPreference='secondaryPreferred',
        appname=os.getenv("OTEL_SERVICE_NAME", "no-name-from-utils")
    )

    return client


def get_data(mongo_db, col_name, ticker, limit=999999999):

    client = get_client()
    db = client[mongo_db]
    history = db[col_name]

    results = history.find({'ticker': ticker},
                           sort=[('datetime', -1)]).limit(limit)
    results_list = list(results)

    if (len(results_list) == 0):
        return []

    data_df = pd.DataFrame(results_list)

    data_df = data_df.sort_values("datetime")

    data_df = data_df.rename(columns={"open": "Open",
                                      "high": "High",
                                      "low": "Low",
                                      "close": "Close"}
                             )

    data_df = data_df.set_index('datetime')

    return data_df


def get_data_by_date(mongo_db: str, 
                     col_name:str , 
                     ticker: str, 
                     start_date: datetime, 
                     end_date=None, 
                     rename=False):
    
        client = get_client()
        db = client[mongo_db]
        history = db[col_name]
    
        if end_date is None:
            end_date = start_date + pd.Timedelta(days=1)
    
        results = history.find({'ticker': ticker,
                                'datetime': {'$gte': start_date,
                                            '$lte': end_date}},
                            sort=[('datetime', -1)])
        results_list = list(results)
    
        if (len(results_list) == 0):
            return []
    
        data_df = pd.DataFrame(results_list)
    
        data_df = data_df.sort_values("datetime")
    
        if rename:
            data_df = data_df.rename(columns={"open": "Open",
                                            "high": "High",
                                            "low": "Low",
                                            "close": "Close"}
                                    )
        
        data_df = data_df.set_index('datetime')
    
        return data_df

def post_results(mongo_db, symbol, test_period, doc, strategy):
    client = get_client()
    db = client[mongo_db]
    col = db['backtest_results']
    col.delete_one({"strategy": strategy,
                                                          "symbol": symbol,
                                                          "period": test_period
                                                          })
    col.update_one(
        {"strategy": strategy,
         "symbol": symbol,
         "period": test_period
         }, {"$set": doc}, upsert=True)
    return True


def post_list_results(mongo_db, symbol, test_period, doc, strategy):
    client = get_client()
    db = client[mongo_db]
    col = db['backtest_results_lists']
    col.delete_one({"strategy": strategy,
                                                                "symbol": symbol,
                                                                "period": test_period
                                                                })

    col.update_one(
        {"strategy": strategy,
         "symbol": symbol,
         "period": test_period
         }, {"$set": doc}, upsert=True)
    return True



import logging

from ..ta import utils
import pkg_resources
import logging

ver = pkg_resources.get_distribution('petrosa').version
logging.info("petrosa-utils version: " + ver)


strategy_list = ['inside_bar_buy',
                 'inside_bar_sell',
                 'continuous_stitch_buy',
                 'continuous_stitch_sell',
                 'setup_91_buy',
                 'setup_91_sell',
                 'setup_92_buy',
                 'setup_92_sell',
                 'setup_93_buy',
                 'setup_93_sell',
                 'setup_94_buy',
                 'setup_94_sell',
                 'fox_trap_buy',
                 'fox_trap_sell',
                 'bear_trap_buy',
                 'bear_trap_sell',
                 'bbss_sell',
                 'bbss_buy',
                 ]

def inside_bar_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(list(dat['Close'])[-1])
    low = float(list(dat['Low'])[-1])
    high = float(list(dat['High'])[-1])

    ema8 = dat["Close"].ewm(span=8, adjust=True, min_periods=7).mean()
    ema80 = dat["Close"].ewm(span=80, adjust=True, min_periods=79).mean()

    ema8 = ema8.iloc[-1]
    ema80 = ema80.iloc[-1]

    last_high = dat.High.iloc[-1]
    prior_to_last_high = dat.High.iloc[-2]

    last_low = dat.Low.iloc[-1]
    prior_to_last_low = dat.Low.iloc[-2]

    if (close > ema8
                and close > ema80
                and last_high < prior_to_last_high
                and last_low > prior_to_last_low
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def inside_bar_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(list(dat['Close'])[-1])
    low = float(list(dat['Low'])[-1])
    high = float(list(dat['High'])[-1])

    ema8 = dat["Close"].ewm(span=8, adjust=True, min_periods=7).mean()
    ema80 = dat["Close"].ewm(span=80, adjust=True, min_periods=79).mean()

    ema8 = ema8.iloc[-1]
    ema80 = ema80.iloc[-1]

    if (close < ema8
            and float(close) < float(ema80)
            and high < float(dat.High.iloc[-2])
            and float(low) > float(dat.Low.iloc[-2])
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def continuous_stitch_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(list(dat['Close'])[-1])
    last_close = float(list(dat['Close'])[-2])
    low = float(list(dat['Low'])[-1])
    high = float(list(dat['High'])[-1])
    high2 = float(list(dat['High'])[-2])
    high3 = float(list(dat['High'])[-3])

    ema20 = dat["Close"].ewm(span=20, min_periods=19, adjust=True).mean()
    ema30 = dat["Close"].ewm(span=30, min_periods=29, adjust=True).mean()
    last_ema20 = ema20.iloc[-2]
    ema20_3 = ema20.iloc[-3]

    ema20 = ema20.iloc[-1]
    ema30 = ema30.iloc[-1]

    if (low <= ema20
                and close > ema20
                and ema20 > ema30
                and high2 > last_ema20
                and high3 > ema20_3
                and last_close > last_ema20
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def continuous_stitch_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    low = float(list(dat['Low'])[-1])
    low2 = float(list(dat['Low'])[-2])
    low3 = float(list(dat['Low'])[-3])
    high = float(list(dat['High'])[-1])
    high1 = float(list(dat['High'])[-2])
    last_close = float(list(dat['Close'])[-2])

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    ema20 = dat["Close"].ewm(span=20, min_periods=19, adjust=True).mean()
    ema30 = dat["Close"].ewm(span=30, min_periods=29, adjust=True).mean()
    last_ema20 = ema20.iloc[-2]
    ema20_3 = ema20.iloc[-3]

    ema9 = ema9.iloc[-1]
    ema20 = ema20.iloc[-1]
    ema30 = ema30.iloc[-1]

    if (low < ema20
                and high >= ema20
                and ema9 < ema20
                and high > high1
                and low2 < last_ema20
                and low3 < ema20_3
                and last_close < last_ema20
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def setup_91_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(dat['Close'].iloc[-1])
    low = float(dat['Low'].iloc[-1])
    high = float(dat['High'].iloc[-1])

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    ema9_last = ema9.iloc[-1]
    ema9_penultimate = ema9.iloc[-2]
    ema9_3 = ema9.iloc[-3]
    last_inclination = ema9_penultimate - ema9_3
    inclination = ema9_last - ema9_penultimate

    if (inclination > 0
            and close > ema9_last
                and last_inclination < 0
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def setup_91_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(dat['Close'].iloc[-1])
    low = float(dat['Low'].iloc[-1])
    high = float(dat['High'].iloc[-1])

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    ema9_last = ema9.iloc[-1]
    ema9_penultimate = ema9.iloc[-2]
    ema9_3 = ema9.iloc[-3]
    last_inclination = ema9_penultimate - ema9_3
    inclination = ema9_last - ema9_penultimate

    if (inclination < 0
        and close < ema9_last
            and last_inclination > 0
        ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def setup_92_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    low = float(dat['Low'].iloc[-1])
    high = float(dat['High'].iloc[-1])

    close = float(dat['Close'].iloc[-1])
    last_low = float(dat['Low'].iloc[-2])

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    inclination = ema9.diff()

    buy_cond_1 = False in list(dat['Low'][-5:].astype(float) > ema9[-5:])
    buy_cond_2 = False in list(inclination[-5:] > 0)

    if (not buy_cond_1 and not buy_cond_2
            and close < last_low
        ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def setup_92_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    low = float(dat['Low'].iloc[-1])
    high = float(dat['High'].iloc[-1])

    close = float(dat['Close'].iloc[-1])
    last_high = float(dat['High'].iloc[-2])

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    inclination = ema9.diff()

    sell_cond_1 = False in list(dat['High'][-5:].astype(float) < ema9[-5:])
    sell_cond_2 = False in list(inclination[-5:] < 0)

    if (not sell_cond_1 and not sell_cond_2
                and close > last_high
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def setup_93_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    inclination = ema9.diff()

    buy_cond_1 = False in list(dat['Low'][-3:].astype(float) > ema9[-3:])
    buy_cond_2 = False in list(inclination[-5:] > 0)

    close_3 = float(dat['Close'].iloc[-3])
    close_2 = float(dat['Close'].iloc[-2])
    close = float(dat['Close'].iloc[-1])
    low = float(dat['Low'].iloc[-1])
    high = float(dat['High'].iloc[-1])

    buy_cond_3 = close_2 < close_3 and close < close_3

    if (not buy_cond_1 and buy_cond_2 and buy_cond_3
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def setup_93_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    inclination = ema9.diff()

    sell_cond_1 = False in list(dat['High'][-3:].astype(float) < ema9[-3:])
    sell_cond_2 = False in list(inclination[-5:] < 0)

    close_3 = float(dat['Close'].iloc[-3])
    close_2 = float(dat['Close'].iloc[-2])
    close = float(dat['Close'].iloc[-1])
    low = float(dat['Low'].iloc[-1])
    high = float(dat['High'].iloc[-1])

    sell_cond_3 = close_2 > close_3 and close > close_3

    if (not sell_cond_1 and sell_cond_2 and sell_cond_3
        ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def setup_94_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    dat['ema'] = ema9
    inc_ema9 = ema9.diff()

    buy_cond_1 = False in list(dat['Low'][-7:-2].astype(float) > ema9[-7:-2])

    close_2 = float(dat['Close'].iloc[-2])
    low = float(dat['Low'].iloc[-1])
    low_2 = float(dat['Low'].iloc[-2])
    high = float(dat['High'].iloc[-1])
    buy_cond_2 = (close_2 < ema9.iloc[-2]) and (low > low_2)
    buy_cond_3 = (inc_ema9.iloc[-2] < 0) and (inc_ema9.iloc[-1] > 0)

    if (not buy_cond_1 and buy_cond_2 and buy_cond_3):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def setup_94_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    ema9 = dat["Close"].ewm(span=9, min_periods=8, adjust=True).mean()
    dat['ema'] = ema9
    inc_ema9 = ema9.diff()

    sell_cond_1 = False in list(dat['High'][-7:-2].astype(float) < ema9[-7:-2])

    close_2 = float(dat['Close'].iloc[-2])
    low = float(dat['Low'].iloc[-1])
    high = float(dat['High'].iloc[-1])
    high_2 = float(dat['High'].iloc[-2])

    # print(inc_ema9)

    sell_cond_2 = (close_2 > ema9.iloc[-2]) and (high < high_2)
    sell_cond_3 = (inc_ema9.iloc[-2] > 0) and (inc_ema9.iloc[-1] < 0)

    if (not sell_cond_1 and sell_cond_2 and sell_cond_3
        ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def fox_trap_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(list(dat['Close'])[-1])
    low = float(list(dat['Low'])[-1])
    high = float(dat['High'].iloc[-1])
    high2 = float(dat['High'].iloc[-2])
    high3 = float(dat['High'].iloc[-3])

    ema8 = dat["Close"].ewm(span=8, min_periods=7, adjust=True).mean()
    ema20 = dat["Close"].ewm(span=20, min_periods=19, adjust=True).mean()
    ema80 = dat["Close"].ewm(span=80, min_periods=79, adjust=True).mean()
    ema20_2 = ema20.iloc[-2]
    ema20_3 = ema20.iloc[-3]

    ema8 = ema8.iloc[-1]
    ema20 = ema20.iloc[-1]
    ema80 = ema80.iloc[-1]

    if (close > ema8
        and close > ema80
        and low < ema8
        and ema8 > ema80
        and ema8 > ema20
        and high2 > ema20_2
            and high3 > ema20_3):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def fox_trap_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(list(dat['Close'])[-1])
    high = float(list(dat['High'])[-1])
    low = float(list(dat['Low'])[-1])
    low2 = float(list(dat['Low'])[-2])
    low3 = float(list(dat['Low'])[-3])

    ema8 = dat["Close"].ewm(span=8, min_periods=7, adjust=True).mean()
    ema80 = dat["Close"].ewm(span=80, min_periods=79, adjust=True).mean()
    ema8_2 = ema8.iloc[-2]
    ema8_3 = ema8.iloc[-3]

    ema8 = ema8.iloc[-1]
    ema80 = ema80.iloc[-1]

    if (close < ema80
            and close < ema8
            and high > ema8
            and ema8 < ema80
            and low2 < ema8_2
            and low3 < ema8_3
            ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def bear_trap_buy(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(list(dat['Close'])[-1])
    last_close = float(list(dat['Close'])[-2])
    low = float(list(dat['Low'])[-1])
    high = float(list(dat['High'])[-1])
    high2 = float(list(dat['High'])[-2])
    high3 = float(list(dat['High'])[-3])

    ema8 = dat["Close"].ewm(span=8, min_periods=7, adjust=True).mean()
    ema80 = dat["Close"].ewm(span=80, min_periods=79, adjust=True).mean()
    last_ema80 = ema80.iloc[-2]
    ema8_2 = ema8.iloc[-2]
    ema8_3 = ema8.iloc[-3]

    ema8 = ema8.iloc[-1]
    ema80 = ema80.iloc[-1]

    if (low < ema80
        and close > ema80
        and last_close > last_ema80
        and high2 > ema8_2
            and high3 > ema8_3):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


def bear_trap_sell(candles, timeframe, periods=126):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    close = float(list(dat['Close'])[-1])
    last_close = float(list(dat['Close'])[-2])
    low = float(list(dat['Low'])[-1])
    low2 = float(list(dat['Low'])[-2])
    low3 = float(list(dat['Low'])[-3])
    high = float(list(dat['High'])[-1])

    ema8 = dat["Close"].ewm(span=8, min_periods=7, adjust=True).mean()
    ema80 = dat["Close"].ewm(span=80, min_periods=79, adjust=True).mean()
    last_ema80 = ema80.iloc[-2]
    ema8_2 = ema8.iloc[-2]
    ema8_3 = ema8.iloc[-3]

    ema8 = ema8.iloc[-1]
    ema80 = ema80.iloc[-1]

    if (high > ema80
        and close < ema80
        and last_close < last_ema80
        and low2 < ema8_2
        and low3 < ema8_3
        ):

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def bbss_sell(candles, timeframe, periods=30):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    low = float(list(dat['Low'])[-1])
    high = float(list(dat['High'])[-1])
    close = float(list(dat['Close'])[-1])



    if not (False in list(dat.Close.iloc[-9:].values > dat.Close.iloc[-13:-4].values)) and dat.Close.iloc[-10] < dat.Close.iloc[-14]:

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=low,
                                        disruption_value=low,
                                        stop_loss=high,
                                        take_profit=low -
                                        ((high - low) * 2),
                                        direction='LOWER')
    else:
        return {}


def bbss_buy(candles, timeframe, periods=30):

    dat = candles

    dat = dat.sort_index(ascending=True)

    if len(dat) < periods:
        logging.info('Error: insufficient data')
        return {}

    low = float(list(dat['Low'])[-1])
    high = float(list(dat['High'])[-1])

    if not (False in list(dat.Close.iloc[-9:].values < dat.Close.iloc[-13:-4].values)) and dat.Close.iloc[-10] > dat.Close.iloc[-14]:

        return utils.strategy_output(ticker=dat.ticker.iloc[-1],
                                        timeframe=timeframe,
                                        pet_datetime=dat.index[-1],
                                        entry_value=high,
                                        disruption_value=high,
                                        stop_loss=low,
                                        take_profit=high +
                                        ((high - low) * 2),
                                        direction='UPPER')
    else:
        return {}


import pandas as pd

from petrosa.checkers import consistency
from petrosa.database import mongo

data = mongo.get_client()["petrosa_crypto"]["candles_h1"].find({"ticker": "BTCUSDT"}).sort("datetime", -1).limit(1010)

df = pd.DataFrame(list(data))

const = consistency.check_consistency(start=min(df["datetime"]), 
                                      end=max(df["datetime"]), 
                                      actual=df["datetime"],
                                      period="h1"
                                      )
print(const)